@@ -32,11 +32,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 '''
 from __future__ import division
 import numpy as np
-import pandas as pd
 from osgeo import gdal, gdalnumeric, ogr, osr
 from gdalconst import *
 from skimage.measure import block_reduce
 import matplotlib.pyplot as plt
+import pandas as pd
 
 # Function to read the original file's projection:
 def get_geo_info(FileName):
@@ -176,7 +176,7 @@ def align_rasters(raster,alignraster,how=np.mean,cxsize=None,cysize=None,masked=
 		GeoT=(max(GeoT1[0],GeoT2[0]), GeoT1[1]*blocksize[0], GeoT1[2], min(GeoT1[3],GeoT2[3]), GeoT1[4] ,GeoT1[-1]*blocksize[1])
 		return (mraster,araster,GeoT)
 	else:
-		print "Rasters need to be in same projection"
+		print("Rasters need to be in same projection")
 		return (-1,-1,-1)
 
 # Load GeoTif raster data
@@ -202,17 +202,36 @@ class GeoRaster():
     '''
     GeoRaster class to create and handle GIS rasters
     Eash GeoRaster object is a numpy masked array + geotransfrom + nodata_value
+    Usage:
+        geo=GeoRaster(raster, geot, nodata_value = ndv)
+    where
+        raster: Numpy masked array with the raster data, which could be loaded with the load_tiff(file)
+        geot: GDAL Geotransformation
+        nodata_value: No data value in raster, optional
     '''
-    def __init__(self, raster, geot, nodata_value = np.nan):
+    def __init__(self, raster, geot, nodata_value = np.nan, projection = None, datatype=None):
+        '''
+        Initialize Georaster
+        Usage:
+            geo=GeoRaster(raster, geot, nodata_value = ndv)
+        where
+            raster: Numpy masked array with the raster data, which could be loaded with the load_tiff(file)
+            geot: GDAL Geotransformation
+            nodata_value: No data value in raster, optional
+        '''
         self.raster = raster
         self.geot = geot
         self.nodata_value = nodata_value
         self.shape = raster.shape
+        self.x_cell_size = geot[1]
+        self.y_cell_size = geot[-1]
         self.xmin = geot[0]
         self.ymax = geot[3]
-        self.xmax = self.xmin + geot[1] * self.shape[1]
-        self.ymin = self.ymax + geot[-1] * self.shape[0]
+        self.xmax = self.xmin + self.x_cell_size * self.shape[1]
+        self.ymin = self.ymax + self.y_cell_size * self.shape[0]
         self.bounds = (self.xmin, self.ymin, self.xmax, self.ymax)
+        self.projection = projection
+        self.datatype=datatype
 
     def __pos__(self):
         return self
@@ -308,12 +327,253 @@ class GeoRaster():
         else:
             return GeoRaster(self.raster**other, self.geot, nodata_value=self.nodata_value)
 
+    def to_tiff(self, filename):
+        '''
+        geo.to_tiff(filename)
+        
+        Saves GeoRaster as GeoTiff filename.tif with type datatype
+        
+        If GeoRaster does not have datatype, then it tries to assign a type.
+        You can assign the type yourself by setting
+         geo.datatype = 'gdal.GDT_'+type
+        '''
+        if self.datatype is None:
+            self.datatype = type(self.raster.data[0,1])
+            if self.datatype.find('int')!=-1:
+                self.datatype = gdal.GDT_Int32
+            else:
+                self.datatype = gdal.GDT_Float64
+        create_geotiff(filename, self.raster, gdal.GetDriverByName('GTiff'), self.nodata_value, self.shape[1], self.shape[0], self.geot, self.projection, self.datatype)
+
     def plot(self):
+        '''
+        geo.plot()
+
+        Returns plot of raster data
+        '''
         plt.matshow(self.raster)
 
     def union(self, other):
+        '''
+        geo.union(Georaster)
+
+        Returns union of GeoRaster with another one
+        '''
         return union([self,other])
 
+    def merge(self, other):
+        '''
+        geo.merge(Georaster)
+
+        Returns merge of GeoRaster with another one
+        '''
+        return merge([self,other])
+
+    def mean(self, *args, **kwargs):
+        '''
+        geo.mean(axis=None, dtype=None, out=None)
+
+        Returns the average of the array elements along given axis.
+
+        Refer to `numpy.mean` for full documentation.
+
+        See Also
+        --------
+        numpy.mean : equivalent function
+        '''
+        return self.raster.mean(*args, **kwargs)
+
+    def max(self, *args, **kwargs):
+        '''
+        geo.max(axis=None, out=None)
+
+        Return the maximum along a given axis.
+
+        Refer to `numpy.amax` for full documentation.
+
+        See Also
+        --------
+        numpy.amax : equivalent function
+        '''
+        return self.raster.max(*args, **kwargs)
+
+    def min(self, *args, **kwargs):
+        '''
+        geo.min(axis=None, out=None)
+
+        Return the minimum along a given axis.
+
+        Refer to `numpy.amin` for full documentation.
+
+        See Also
+        --------
+        numpy.amin : equivalent function
+        '''
+        return self.raster.min(*args, **kwargs)
+
+    def median(self, *args, **kwargs):
+        '''
+        geo.median(axis=None, out=None, overwrite_input=False)
+        
+        axis : int, optional
+            Axis along which the medians are computed. The default (axis=None)
+            is to compute the median along a flattened version of the array.
+        out : ndarray, optional
+            Alternative output array in which to place the result. It must have
+            the same shape and buffer length as the expected output, but the
+            type (of the output) will be cast if necessary.
+        overwrite_input : bool, optional
+           If True, then allow use of memory of input array (a) for
+           calculations. The input array will be modified by the call to
+           median. This will save memory when you do not need to preserve the
+           contents of the input array. Treat the input as undefined, but it
+           will probably be fully or partially sorted. Default is False. Note
+           that, if `overwrite_input` is True and the input is not already an
+           ndarray, an error will be raised.
+        '''
+        return np.median(self.raster.max, *args, **kwargs)
+
+    def std(self, *args, **kwargs):
+        '''
+        geo.std(axis=None, dtype=None, out=None, ddof=0)
+
+        Returns the standard deviation of the array elements along given axis.
+
+        Refer to `numpy.std` for full documentation.
+
+        See Also
+        --------
+        numpy.std : equivalent function
+        '''
+        return self.raster.std(*args, **kwargs)
+
+    def argmax(self, *args, **kwargs):
+        '''
+        geo.argmax(axis=None, out=None)
+
+        Return indices of the maximum values along the given axis.
+
+        Refer to `numpy.argmax` for full documentation.
+
+        See Also
+        --------
+        numpy.argmax : equivalent function
+        '''
+        return self.raster.argmax(*args, **kwargs)
+
+    def argmin(self, *args, **kwargs):
+        '''
+        geo.argmin(axis=None, out=None)
+
+        Return indices of the minimum values along the given axis of `a`.
+
+        Refer to `numpy.argmin` for detailed documentation.
+
+        See Also
+        --------
+        numpy.argmin : equivalent function
+        '''
+        return self.raster.argmin(*args, **kwargs)
+
+    def sum(self, *args, **kwargs):
+        '''
+        geo.sum(axis=None, dtype=None, out=None)
+
+        Return the sum of the array elements over the given axis.
+
+        Refer to `numpy.sum` for full documentation.
+
+        See Also
+        --------
+        numpy.sum : equivalent function
+        '''
+        return self.raster.sum(*args, **kwargs)
+
+    def prod(self, *args, **kwargs):
+        '''
+        geo.prod(axis=None, dtype=None, out=None)
+
+        Return the product of the array elements over the given axis
+
+        Refer to `numpy.prod` for full documentation.
+
+        See Also
+        --------
+        numpy.prod : equivalent function
+        '''
+        return self.raster.prod(*args, **kwargs)
+
+    def var(self, *args, **kwargs):
+        '''
+        geo.var(axis=None, dtype=None, out=None, ddof=0)
+
+        Returns the variance of the array elements, along given axis.
+
+        Refer to `numpy.var` for full documentation.
+
+        See Also
+        --------
+        numpy.var : equivalent function
+        '''
+        return self.raster.var(*args, **kwargs)
+
+    def gini(self):
+        """Return computed Gini coefficient.
+        """
+        xsort = sorted(self.raster.data[self.raster.data!=self.nodata_value].flatten()) # increasing order
+        y = np.cumsum(xsort)
+        B = sum(y) / (y[-1] * len(xsort))
+        return 1 + 1./len(xsort) - 2*B
+
+    def flatten(self, *args, **kwargs):
+        '''
+        geo.flatten(order='C')
+
+        Return a copy of the array collapsed into one dimension.
+
+        Parameters
+        ----------
+        order : {'C', 'F', 'A'}, optional
+            Whether to flatten in C (row-major), Fortran (column-major) order,
+            or preserve the C/Fortran ordering from `a`.
+            The default is 'C'.
+        '''
+        return self.raster.flatten(*args, **kwargs)
+
+    def apply(self, func, *args, **kwargs):
+        '''
+        geo.apply(func, *args, **kwargs)
+        
+        Returns the value of applying function func on the raster data
+        
+        func: Python function
+        *args: Arguments of function
+        **kwargs: Additional arguments of function
+        '''
+        return func(self.raster, *args, **kwargs)
+
+    def map_pixel(self, point_x, point_y):
+        '''
+        geo.map_pixel(point_x, point_y)
+        
+        Return value of raster in location 
+        '''
+        col, row =map_pixel(point_x, point_y, self.x_cell_size, self.y_cell_size, self.xmin, self.ymax)
+        return self.raster[row, col]
+
+    def extract(self, point_x, point_y, radius=0):
+        '''
+        geo.extract(x, y, radius=r)
+        
+        Return subraster of raster geo around location (x,y) with radius r
+        where (x,y) and r are in the same coordinate system as geo
+        '''
+        col, row =map_pixel(point_x, point_y, self.x_cell_size, self.y_cell_size, self.xmin, self.ymax)
+        col2 = self.x_cell_size/radius
+        row2 = self.y_cell_size/radius
+        return GeoRaster(self.raster[max(row-row2, 0):min(row+row2, self.shape[0]), \
+                        max(col-col2, 0):min(col+col2, self.shape[1])], self.geot, nodata_value = self.nodata_value)
+
 # Union of rasters
 def union(rasters):
     """
@@ -323,7 +583,7 @@ def union(rasters):
     where
         rasters is a list of GeoRaster objects
     """
-    if sum([rasters[0].geot[1]==i.geot[1] for i in rasters])==len(rasters) and sum([rasters[0].geot[-1]==i.geot[-1] for i in rasters])==len(rasters):
+    if sum([rasters[0].x_cell_size==i.x_cell_size for i in rasters])==len(rasters) and sum([rasters[0].y_cell_size==i.y_cell_size for i in rasters])==len(rasters):
         if sum([rasters[0].nodata_value==i.nodata_value for i in rasters])==len(rasters):
             ndv=rasters[0].nodata_value
         else:
@@ -332,12 +592,13 @@ def union(rasters):
         lonmax = max([i.xmax for i in rasters])
         latmin = min([i.ymin for i in rasters])
         latmax = max([i.ymax for i in rasters])
-        shape = (np.round((latmax-latmin)/rasters[0].geot[1]).astype(int),np.round((lonmax-lonmin)/rasters[0].geot[1]).astype(int))
+        shape = (np.round((latmax-latmin)/rasters[0].x_cell_size).astype(int),np.round((lonmax-lonmin)/rasters[0].x_cell_size).astype(int))
         out = ndv*np.ones(shape)
         for i in rasters:
-            (col,row) = map_pixel(i.xmin, i.ymax, rasters[0].geot[1], rasters[0].geot[-1], lonmin, latmax)
-            out[row:row+i.shape[0],col:col+i.shape[1]] = np.where(i.raster.data!=i.nodata_value, i.raster.data, out[row:row+i.shape[0],col:col+i.shape[1]])#i.raster
-        return GeoRaster(out, (lonmin, rasters[0].geot[1], 0.0, latmax, 0.0, rasters[0].geot[-1]), nodata_value=ndv)
+            (col,row) = map_pixel(i.xmin, i.ymax, rasters[0].x_cell_size, rasters[0].y_cell_size, lonmin, latmax)
+            out[row:row+i.shape[0],col:col+i.shape[1]] = np.where(i.raster.data!=i.nodata_value, i.raster.data,\
+                                                         out[row:row+i.shape[0],col:col+i.shape[1]])#i.raster
+        return GeoRaster(out, (lonmin, rasters[0].x_cell_size, 0.0, latmax, 0.0, rasters[0].y_cell_size), nodata_value=ndv)
     else:
         raise RasterGeoError('Rasters need to have same pixel sizes. Use the aggregate or dissolve functions to generate correct GeoRasters')
 
@@ -351,3 +612,12 @@ def merge(rasters):
     """
     return union(rasters)
 
+# Load data into a GeoRaster from file
+def from_file(filename):
+    """
+    Create a GeoRaster object from a file
+    """
+    NDV, xsize, ysize, GeoT, Projection, DataType = get_geo_info(filename)
+    data = gdalnumeric.LoadFile(filename)
+    data = np.ma.masked_array(data, mask=data==NDV,fill_value=NDV)
+    return GeoRaster(data,GeoT, nodata_value=NDV, projection=Projection, datatype=DataType)
